@@ -3,6 +3,8 @@ import copy
 import gffutils
 import itertools
 
+import numpy as np
+
 from collections import defaultdict
 from functools import reduce
 
@@ -15,12 +17,13 @@ def createDatabase (gff_file, remove_pseudogenes = True):
 
 		# Delete any pseudogenes, if specified
 		if remove_pseudogenes:
+			pseudogene_gene_features = [feature for feature in gff_db.features_of_type('pseudogene')]
+			pseudogenes_children_features = list(itertools.chain.from_iterable(gff_db.children(_pgf) for _pgf in pseudogene_gene_features))
 			try:
-				pseudogenes = [feature.attributes['gene'][0] for feature in gff_db.features_of_type('pseudogene')]
-				pseudogenes_intron_features = [feature for feature in gff_db.features_of_type('intron') if feature.attributes['transcript_id'][0] in pseudogenes]
-				pseudogenes_w_exons_features = list(itertools.chain.from_iterable(gff_db.iter_by_parent_childs(featuretype='pseudogene')))
-				gff_db.delete(pseudogenes_w_exons_features + pseudogenes_intron_features, make_backup = False)
-			except: pass
+				pseudogene_tids = [_pgf.attributes['gene'][0] for _pgf in pseudogene_gene_features]
+				pseudogenes_tid_features = [feature for feature in gff_db.features_of_type('intron') if feature.attributes['transcript_id'][0] in pseudogene_tids]
+			except: pseudogenes_tid_features = []
+			gff_db.delete(pseudogene_gene_features + pseudogenes_children_features + pseudogenes_tid_features, make_backup = False)
 
 	# Read the database, if already found
 	else: gff_db = gffutils.FeatureDB(gff_db_filename)
@@ -294,7 +297,7 @@ def createIntrons (gff_db, gene, label_first_intron):
 		intron_intervals = subtractItervals(intron_intervals, exon_intervals)
 
 		# Confirm the order of the intervals
-		if mRNA.strand == '-': intron_intervals = sorted(intron_intervals, key=lambda x: x[0], reverse = True)
+		if mRNA.strand == '-': intron_intervals = sorted(intron_intervals, key = lambda x: x[0], reverse = True)
 
 		# Create the intron GFF entires for the mRNA, if the
 		# mRNA is on the reverse strand. Reverse the intron
@@ -557,7 +560,7 @@ def chromCounts (gff_db, chrom_name, chrom_limit, feature_count_dict, feature_se
 	the relevant size is specified by the user. Update the gene
 	intervals to include the additional features.
 	'''
-	for gene in gff_db.features_of_type(['gene', 'rRNA', 'tRNA'], limit=f'{chrom_name}:1-{chrom_limit}'):
+	for gene in gff_db.features_of_type('gene', limit=f'{chrom_name}:1-{chrom_limit}'):
 		gene_interval = [copy.deepcopy(gene.start), copy.deepcopy(gene.end)]
 		if promoter_bp:
 			gene_interval, prime5_interval = assignPrime5(promoter_bp, gene, gene_interval, chrom_limit)
@@ -592,7 +595,7 @@ def chromCounts (gff_db, chrom_name, chrom_limit, feature_count_dict, feature_se
 		if downstream_bp: feature_dict['downstream'] = returnItervalOverlaps(added_feature_dict['downstream'], (merged_interval_start, merged_interval_end))
 		for feature in features: feature_dict[feature.featuretype].append((feature.start, feature.end))
 		for feature in feature_dict.keys(): feature_dict[feature] = mergeItervals(feature_dict[feature])
-		
+
 		# Check if the intervals should be prioritized
 		if prioritize:
 
@@ -614,14 +617,12 @@ def chromCounts (gff_db, chrom_name, chrom_limit, feature_count_dict, feature_se
 		'''
 		for feature in feature_dict.keys(): feature_count_dict = countItervals(feature_dict[feature], feature, feature_count_dict)
 		for feature in feature_dict.keys():
-			if feature in ['promoter', 'upstream', 'downstream'] and not prioritize: continue
-			#print(intergenic_interval)
+			if feature in ['promoter', 'upstream', 'downstream', 'tss_flanks', 'tss_upstream'] and not prioritize: continue
 			intergenic_interval = subtractItervals(intergenic_interval, feature_dict[feature])
 		feature_count_dict = countItervals(intergenic_interval, 'intergenic', feature_count_dict)
 
 	# Bulk count the intergenic sites at the 3', if needed
 	if chromosome_pos <= chrom_limit: feature_count_dict['intergenic'] += ((chrom_limit + 1) - chromosome_pos)
-
 	return feature_count_dict
 
 def posCounts (gff_db, chrom_name, chrom_limit, chrom_position_list, feature_set, prioritize = False, priority_order = None, promoter_bp = None, upstream_bp = None, downstream_bp = None, **kwargs):
@@ -646,6 +647,29 @@ def posCounts (gff_db, chrom_name, chrom_limit, chrom_position_list, feature_set
 	the genes associated with each position/feature are recorded. 
 	'''
 
+	def returnParents (feature):
+		
+		# Assign parent(s) of the feature
+		parent_gene = ''
+		parent_other = []
+		for parent_feature in gff_db.parents(feature):
+			if parent_feature.featuretype == 'gene':
+				if 'Name' in parent_feature.attributes: parent_gene = parent_feature.attributes['Name'][0]
+				else: parent_gene = parent_feature.id
+			else:
+				if 'Name' in parent_feature.attributes: parent_other.append(parent_feature.attributes['Name'][0])
+				else: parent_other.append(parent_feature.id)
+
+		# Return gene and sub-parent (if found)
+		if parent_other: return f'{parent_gene}({",".join(parent_other)})'
+		else: return parent_gene
+
+	def yieldPositions (position_list):
+		for _cpos in position_list:
+			if isinstance(_cpos, np.int64): yield (_cpos, _cpos)
+			elif isinstance(_cpos, np.ndarray): yield (_cpos[0], _cpos[1])
+			else: raise Exception(f'Unable to yield from type({type(_cpos)}) with value: {_cpos}')
+
 	'''
 	Assign a dict to store the features and genes for each 
 	position
@@ -696,7 +720,11 @@ def posCounts (gff_db, chrom_name, chrom_limit, chrom_position_list, feature_set
 		Assign the relevant positions - i.e. positions within the
 		merged interval.
 		'''
-		relevant_positions = [_cpos for _cpos in chrom_position_list if (merged_interval_start <= _cpos and _cpos <= merged_interval_end)]
+		relevant_positions = []
+		for chrom_position_start, chrom_position_stop in yieldPositions(chrom_position_list):
+			if chrom_position_start <= merged_interval_end and merged_interval_start <= chrom_position_stop: relevant_positions.append((chrom_position_start, chrom_position_stop))
+
+		# Assign the starting intergenic interval
 		intergenic_interval = [(merged_interval_start, merged_interval_end)]
 
 		'''
@@ -738,28 +766,28 @@ def posCounts (gff_db, chrom_name, chrom_limit, chrom_position_list, feature_set
 		'''
 		for feature_type, feature_intervals in feature_dict.items():
 			for feature_interval_start, feature_interval_end in feature_intervals:
-				for relevant_position in relevant_positions:
-					if (feature_interval_start <= relevant_position and relevant_position <= feature_interval_end):
+				for relevant_position_start, relevant_position_stop in relevant_positions:
+					if relevant_position_start <= feature_interval_end and feature_interval_start <= relevant_position_stop:
 						if feature_type in list(added_feature_dict):
 							for gene_id, added_feature_intervals, in added_feature_dict[feature_type].items():
 								for added_feature_start, added_feature_end in added_feature_intervals:
-									if (added_feature_start <= relevant_position and relevant_position <= added_feature_end):
-										position_feature_dict[relevant_position].append([feature_type, gene_id])
+									if relevant_position_start <= added_feature_end and added_feature_start <= relevant_position_stop:
+										position_feature_dict[(relevant_position_start, relevant_position_stop)].append([feature_type, gene_id])
 						else:
 							for feature in features:
 								if feature.featuretype != feature_type: continue
-								if (feature.start <= relevant_position and relevant_position <= feature.end):
-									if 'transcript_id' in feature.attributes: position_feature_dict[relevant_position].append([feature_type, feature.attributes['transcript_id'][0]])
-									elif 'Parent' in feature.attributes: position_feature_dict[relevant_position].append([feature_type, feature.attributes['Parent'][0]])
+								if relevant_position_start <= feature.end and feature.start <= relevant_position_stop:
+									if 'transcript_id' in feature.attributes: position_feature_dict[(relevant_position_start, relevant_position_stop)].append([feature_type, returnParents(feature)])
+									elif 'Parent' in feature.attributes: position_feature_dict[(relevant_position_start, relevant_position_stop)].append([feature_type, returnParents(feature)])
 									elif feature_type in ['tss_flanks', 'tss_upstream']:
-										position_feature_dict[relevant_position].append([feature_type, feature.attributes['ID'][0].split(':')[0]])
+										position_feature_dict[(relevant_position_start, relevant_position_stop)].append([feature_type, feature.attributes['ID'][0].split(':')[0]])
 									else: raise Exception(f'Unable to assign {feature_type} with attributes {feature.attributes}')
 
 	'''
 	Lastly, assign the intergenic feature for all positions 
 	outside the merged intervals and return the feature dict
 	'''
-	for chrom_position in chrom_position_list:
+	for chrom_position in yieldPositions(chrom_position_list):
 		if chrom_position not in position_feature_dict:
 			position_feature_dict[chrom_position].append(['intergenic', ''])
 	return position_feature_dict
